use phf::{phf_map, Map};

pub const TRUNCATE_LEVEL_DATA: bool = true;

pub const YOUTUBE_CHANNEL_BASE_URL: &str = "https://www.youtube.com/channel/";

pub const XOR_KEY: u8 = 0xB;
pub const AES_KEY: [u8; 32] = [
    0x69, 0x70, 0x75, 0x39, 0x54, 0x55, 0x76, 0x35, 0x34, 0x79, 0x76, 0x5d, 0x69, 0x73, 0x46, 0x4d,
    0x68, 0x35, 0x40, 0x3b, 0x74, 0x2e, 0x35, 0x77, 0x33, 0x34, 0x45, 0x32, 0x52, 0x79, 0x40, 0x7b,
];
// pub const PASSWORD_SALT: &str = "mI29fmAnxgTs";

#[derive(Copy, Clone)]
pub enum ArrayInnerType {
    // ! String, Uncomment if needed
    Number,
}

#[derive(Copy, Clone)]
pub enum ValueType {
    Array(char, ArrayInnerType),
    ChunkMap(char),
}

pub static KEY_VALUE_TYPES: Map<&'static str, ValueType> = phf_map! {
    "extra_artists" => ValueType::Array('.', ArrayInnerType::Number),
    "used_sfx" => ValueType::Array(',', ArrayInnerType::Number),
    "used_songs" => ValueType::Array(',', ArrayInnerType::Number),
    "capacity_string" => ValueType::Array(',', ArrayInnerType::Number),
    "scores" => ValueType::Array(',', ArrayInnerType::Number),
    "dpad01" => ValueType::Array(',', ArrayInnerType::Number),
    "dpad02" => ValueType::Array(',', ArrayInnerType::Number),
    "dpad03" => ValueType::Array(',', ArrayInnerType::Number),
    "dpad04" => ValueType::Array(',', ArrayInnerType::Number),
    "dpad05" => ValueType::Array(',', ArrayInnerType::Number),
    "extra_artist_names" => ValueType::ChunkMap(','),
};

pub const BOOLEAN_PARENT_KEYS: [&str; 8] = [
    "likes",
    "bronze_user_coins",
    "completed_levels",
    "wraith_rewards",
    "user_coins",
    "followed_accounts",
    "enabled_search_filters",
    "enabled_items",
];
pub const BOOLEAN_KEYS: [&str; 3] = ["level_page_coin", "glubfub_coin", "nong"];

pub static GAMESAVE_KEYS: Map<&'static str, &'static str> = phf_map! {
    "GS_value" => "stats",
    "GS_completed" => "completed_levels",
    "GS_3" => "user_coins",
    "GS_4" => "bronze_user_coins",
    "GS_5" => "map_pack_stars",
    "GS_6" => "shop_purchases",
    "GS_7" => "level_progress",
    "GS_8" => "unknown_1",
    "GS_9" => "level_stars",
    "GS_10" => "official_level_progress",
    "GS_11" => "daily_rewards",
    "GS_12" => "quests",
    "GS_13" => "unknown_2",
    "GS_14" => "quest_rewards",
    "GS_15" => "queued_quests",
    "GS_16" => "daily_progress",
    "GS_17" => "daily_stars",
    "GS_18" => "gauntlet_progress",
    "GS_19" => "treasure_room_rewards",
    "GS_20" => "total_demon_keys",
    "GS_21" => "rewards",
    "GS_22" => "gd_world_rewards",
    "GS_23" => "gauntlet_progress_2",
    "GS_24" => "daily_progress_2",
    "GS_25" => "weekly_rewards",
    "GS_26" => "active_path",
    "GS_27" => "completed_lists",
    "GS_28" => "enabled_items",
    "GS_30" => "wraith_rewards",
    "GS_31" => "event_rewards",
    "GLM_01" => "official_levels",
    "GLM_02" => "uploaded_levels",
    "GLM_03" => "online_levels",
    "GLM_04" => "starred_levels",
    "GLM_05" => "unknown_3",
    "GLM_06" => "followed_accounts",
    "GLM_07" => "recently_played",
    "GLM_08" => "enabled_search_filters",
    "GLM_09" => "available_search_filters",
    "GLM_10" => "timely_levels",
    "GLM_11" => "daily_id",
    "GLM_12" => "likes",
    "GLM_13" => "rated_levels",
    "GLM_14" => "reported_levels",
    "GLM_15" => "rated_demons",
    "GLM_16" => "gauntlets",
    "GLM_17" => "weekly_id",
    "GLM_18" => "level_folders",
    "GLM_19" => "created_level_folders",
    "GLM_20" => "smart_templates",
    "GLM_22" => "favourited_lists",
    "GLM_23" => "event_id",
    "GJA_001" => "username",
    "GJA_002" => "password",
    "GJA_003" => "account_id",
    "GJA_004" => "session_id",
    "GJA_005" => "hashed_password",
    "LLM_01" => "local_levels",
    "LLM_02" => "binary_version",
    "LLM_03" => "local_lists",
    "MDLM_001" => "song_info",
    "MDLM_002" => "song_priority",
    "MDLM_003" => "unknown_4",
    "KBM_001" => "keybinds",
    "KBM_002" => "keybinds_2",
    "texQuality" => "texture_quality",
    "customObjectDict" => "custom_objects",
    "playerUserID" => "player_id",
    "playerUDID" => "player_udid",
    "playerName" => "player_name",
    "playerBall" => "player_ball",
    "playerBird" => "player_ufo",
    "playerDart" => "player_wave",
    "playerRobot" => "player_robot",
    "playerColor2" => "player_colour_2",
    "playerColor3" => "player_colour_3",
    "playerShip" => "player_ship",
    "playerSpider" => "player_spider",
    "playerDeathEffect" => "player_death_effect",
    "playerFrame" => "player_frame",
    "playerJetpack" => "player_jetpack",
    "playerShipStreak" => "player_ship_streak",
    "playerStreak" => "player_streak",
    "playerSwing" => "player_swing",
    "practiceOpacity" => "practice_opacity",
    "practicePosX" => "practice_pos_x",
    "practicePosY" => "practice_pos_y",
    "reportedAchievements" => "achievements",
    "secretNumber" => "cod3breaker_solution",
    "clickedGarage" => "clicked_icon_kit",
    "hasRP" => "is_mod",
    "kCEK" => "type",
    "valueKeeper" => "game_values",
    "unlockValueKeeper" => "ingame_events",
    "sfxVolume" => "sfx_volume",
    "bgVolume" => "bg_volume",
    "binaryVersion" => "binary_version",
    "customFPSTarget" => "fps_target",
};

pub static DIFFICULTY: Map<&'static str, &'static str> = phf_map! {
    "-1" => "n/a",
    "0" => "auto",
    "1" => "easy",
    "2" => "normal",
    "3" => "hard",
    "4" => "harder",
    "5" => "insane",
    "6" => "hard_demon",
    "7" => "easy_demon",
    "8" => "medium_demon",
    "9" => "insane_demon",
    "10" => "extreme_demon"
};

pub const REWARD_KEYS: Map<&'static str, &'static str> = phf_map! {
    "1" => "item",
    "2" => "icon_id",
    "3" => "amount",
    "4" => "icon_form"
};

pub const REWARD_DATA_KEYS: Map<&'static str, &'static str> = phf_map! {
    "1" => "item_type",
    "2" => "custom_item_id",
    "3" => "amount",
    "4" => "item_unlock_value",
};

pub static LEVEL_TYPE: Map<&'static str, &'static str> = phf_map! {
    "1" => "official",
    "2" => "local",
    "3" => "saved",
    "4" => "online"
};

pub static LEVEL_LENGTH: Map<&'static str, &'static str> = phf_map! {
    "0" => "tiny",
    "1" => "short",
    "2" => "medium",
    "3" => "long",
    "4" => "xl",
    "5" => "platformer"
};

pub static EPIC_RATING: Map<&'static str, &'static str> = phf_map! {
    "0" => "none",
    "1" => "epic",
    "2" => "legendary",
    "3" => "mythic"
};

pub static LIST_KEYS: Map<&'static str, &'static str> = phf_map! {
    "k1" => "id",
    "k2" => "name",
    "k3" => "description",
    "k5" => "level_creator",
    "k7" => "difficulty",
    "k11" => "downloads",
    "k15" => "uploaded",
    "k16" => "version",
    "k21" => "list_type",
    "k22" => "rating",
    "k25" => "is_demon",
    "k26" => "stars",
    "k27" => "featured",
    "k42" => "original_list_id",
    "k46" => "version",
    "k60" => "account_id",
    "k79" => "unlisted",
    "k82" => "favourited",
    "k83" => "order",
    "k94" => "unknown_1",
    "k96" => "level_ids",
    "k97" => "levels",
    "k98" => "upload_date",
    "k99" => "update_date",
    "k100" => "unknown_2",
    "k113" => "diamond_reward",
    "k114" => "level_count_threshold",
};

pub static LEVEL_KEYS: Map<&'static str, &'static str> = phf_map! {
    "k1" => "id",
    "k2" => "name",
    "k3" => "description",
    "k4" => "level_data",
    "k5" => "author",
    "k6" => "player_id",
    "k7" => "difficulty",
    "k8" => "official_song_id",
    "k9" => "rating_score_1",
    "k10" => "rating_score_2",
    "k11" => "downloads",
    "k12" => "completions",
    "k13" => "editable",
    "k14" => "verified",
    "k15" => "uploaded",
    "k16" => "version",
    "k17" => "game_version",
    "k18" => "attempts",
    "k19" => "percentage",
    "k20" => "practice_percentage",
    "k21" => "level_type",
    "k22" => "likes",
    "k23" => "length",
    "k24" => "dislikes",
    "k25" => "demon",
    "k26" => "stars",
    "k27" => "featured_position",
    "k33" => "auto",
    "k34" => "replay_data",
    "k35" => "playable",
    "k36" => "jumps",
    "k37" => "secret_coins_to_unlock",
    "k38" => "level_unlocked",
    "k39" => "level_size",
    "k40" => "build_version",
    "k41" => "password",
    "k42" => "copied_id",
    "k43" => "two_player",
    "k45" => "custom_song_id",
    "k46" => "revision",
    "k47" => "edited_outside_editor",
    "k48" => "objects",
    "k50" => "binary_version",
    "k60" => "account_id",
    "k61" => "first_coin_collected",
    "k62" => "second_coin_collected",
    "k63" => "third_coin_collected",
    "k64" => "total_coins",
    "k65" => "verified_coins",
    "k66" => "requested_stars",
    "k67" => "capacity_string",
    "k68" => "anti_cheat_triggered",
    "k69" => "high_object_count",
    "k71" => "mana_orb_percentage",
    "k72" => "ldm",
    "k73" => "ldm_enabled",
    "k74" => "timely_id",
    "k75" => "epic_rating",
    "k76" => "demon_type",
    "k77" => "is_gauntlet",
    "k78" => "is_gauntlet_2",
    "k79" => "unlisted",
    "k80" => "editor_time",
    "k81" => "total_editor_time",
    "k82" => "favourited",
    "k83" => "saved_level_index",
    "k84" => "folder",
    "k85" => "clicks",
    "k86" => "best_attempt_time",
    "k87" => "seed",
    "k88" => "scores",
    "k89" => "leaderboard_valid",
    "k90" => "leaderboard_percentage",
    "k91" => "unknown_1",
    "k92" => "unknown_2",
    "k93" => "unlimited_objects_?",
    "k94" => "platformer_?",
    "k95" => "ticks_to_complete",
    "k101" => "unknown_3",
    "k104" => "used_songs",
    "k105" => "used_sfx",
    "k106" => "unknown_4",
    "k107" => "best_time",
    "k108" => "best_points",
    "k109" => "local_times",
    "k110" => "local_scores",
    "k111" => "platformer_seed",
    "k112" => "disable_shake",
    "kI1" => "editor_camera_x",
    "kI2" => "editor_camera_y",
    "kI3" => "editor_camera_zoom",
    "kI4" => "editor_build_tab_page",
    "kI5" => "editor_build_tab_category",
    "kI6" => "editor_recent_pages",
    "kI7" => "editor_layer"
};

pub const KCEK_KEYS: Map<&'static str, &'static str> = phf_map! {
    "4" => "level",
    "6" => "song",
    "7" => "quest",
    "8" => "reward",
    "9" => "reward_data",
    "10" => "smart_templates",
    "11" => "smart_template_variation",
    "12" => "lists"
};

pub const SMART_TEMPLATE_KEYS: Map<&'static str, &'static str> = phf_map! {
    "1" => "id",
    "2" => "name",
    "3" => "variations",
    "4" => "unknown_1",
    "5" => "unknown_2",
    "6" => "unknown_3",
    "7" => "unknown_4",
};

pub const STAT_KEYS: Map<&'static str, &'static str> = phf_map! {
    "1" => "jumps",
    "2" => "attempts",
    "3" => "official_completed_levels",
    "4" => "online_completed_levels",
    "5" => "completed_demons",
    "6" => "stars",
    "7" => "completed_map_packs",
    "8" => "secret_coins",
    "9" => "destroyed_players",
    "10" => "liked_levels",
    "11" => "rated_levels",
    "12" => "user_coins",
    "13" => "diamonds",
    "14" => "orbs",
    "15" => "completed_dailies",
    "16" => "fire_shards",
    "17" => "ice_shards",
    "18" => "poison_shards",
    "19" => "shadow_shards",
    "20" => "lava_shards",
    "21" => "demon_keys",
    "22" => "total_orbs",
    "23" => "earth_shards",
    "24" => "blood_shards",
    "25" => "metal_shards",
    "26" => "light_shards",
    "27" => "soul_shards",
    "28" => "moons",
    "29" => "spendable_diamonds",
    "30" => "fire_path",
    "31" => "ice_path",
    "32" => "poison_path",
    "33" => "shadow_path",
    "34" => "lava_path",
    "35" => "earth_path",
    "36" => "blood_path",
    "37" => "metal_path",
    "38" => "light_path",
    "39" => "soul_path",
    "40" => "gauntlets",
    "41" => "list_rewards",
    "42" => "insane_levels_completed",
    "43" => "event_levels_completed",
    "unique_secretB03" => "glubfub_coin",
    "unique_secret04" => "level_page_coin",
    "unique_secret06" => "sparky_coin",
};

pub const QUEST_TYPE: Map<&'static str, &'static str> = phf_map! {
    "1" => "orbs",
    "2" => "coins",
    "3" => "stars",
};

pub const QUEST_KEYS: Map<&'static str, &'static str> = phf_map! {
    "1" => "type",
    "2" => "obtained_items",
    "3" => "required_items",
    "4" => "diamonds",
    "5" => "time_left",
    "6" => "active",
    "7" => "name",
    "8" => "tier"
};

pub const ITEMS: Map<&'static str, &'static str> = phf_map! {
    "1" => "fire_shards",
    "2" => "ice_shards",
    "3" => "poison_shards",
    "4" => "shadow_shards",
    "5" => "lava_shards",
    "6" => "demon_key",
    "7" => "orbs",
    "8" => "diamonds",
    "9" => "icon",
    "10" => "earth_shards",
    "11" => "blood_shards",
    "12" => "metal_shards",
    "13" => "light_shards",
    "14" => "soul_shards",
    "15" => "golden_keys"
};

pub const ICON_FORMS: Map<&'static str, &'static str> = phf_map! {
    "1" => "cube",
    "2" => "colour_1",
    "3" => "colour_2",
    "4" => "ship",
    "5" => "ball",
    "6" => "ufo",
    "7" => "wave",
    "8" => "robot",
    "9" => "spider",
    "10" => "trail",
    "11" => "death_effect",
    "12" => "item",
    "13" => "swing",
    "14" => "jetpack",
    "15" => "ship_fire"
};

pub const CHEST_IDS: Map<&'static str, &'static str> = phf_map! {
    "1" => "4_hour",
    "2" => "24_hour",
    "3" => "1_key",
    "4" => "5_key",
    "5" => "10_key",
    "6" => "25_key",
    "7" => "50_key",
    "8" => "100_key"
};

pub const SONG_KEYS: Map<&'static str, &'static str> = phf_map! {
    "1" => "id",
    "2" => "name",
    "3" => "artist_id",
    "4" => "artist",
    "5" => "size_mb",
    "6" => "youtube_id",
    "7" => "youtube_channel",
    "8" => "verified",
    "9" => "priority",
    "10" => "mp3_url",
    "11" => "nong",
    "12" => "extra_artists",
    "13" => "new",
    "14" => "new_type",
    "15" => "extra_artist_names"
};

pub const SPECIAL_SHOP_PURCHASES: Map<&'static str, &'static str> = phf_map! {
    "187" => "menu_music_customizer",
    "188" => "practice_music_unlocker",
    "244" => "robot_animation_slow",
    "245" => "robot_animation_fast",
    "246" => "spider_animation_fast"
};

pub const GAME_VARIABLES: Map<&'static str, &'static str> = phf_map! {
    "gv_0001" => "editor.follow_player",
    "gv_0002" => "editor.play_music",
    "gv_0003" => "editor.swipe",
    "gv_0004" => "editor.free_move",
    "gv_0005" => "editor.delete_filter",
    "gv_0006" => "editor.delete_object_id",
    "gv_0007" => "editor.rotate",
    "gv_0008" => "editor.snap",
    "gv_0009" => "editor.ignore_damage",
    "gv_0010" => "flip2_player_controls",
    "gv_0011" => "always_limit_controls",
    "gv_0012" => "accepted_comment_rules",
    "gv_0013" => "increase_max_undo",
    "gv_0014" => "disable_explosion_shake",
    "gv_0015" => "flip_pause_button",
    "gv_0016" => "accepted_song_tos",
    "gv_0018" => "no_song_limit",
    "gv_0019" => "load_songs_to_memory",
    "gv_0022" => "higher_audio_quality",
    "gv_0023" => "smooth_fix",
    "gv_0024" => "show_cursor",
    "gv_0025" => "fullscreen",
    "gv_0026" => "auto_retry",
    "gv_0027" => "auto_checkpoints",
    "gv_0028" => "disable_thumbstick",
    "gv_0029" => "showed_upload_popup",
    "gv_0030" => "vsync",
    "gv_0031" => "unused.call_gl_finish",
    "gv_0032" => "unused.force_timer_enabled",
    "gv_0033" => "change_song_location",
    "gv_0034" => "game_center",
    "gv_0035" => "small_grid_step",
    "gv_0036" => "editor.preview_mode",
    "gv_0037" => "editor.show_ground",
    "gv_0038" => "editor.show_grid",
    "gv_0039" => "editor.grid_on_top",
    "gv_0040" => "show_percentage",
    "gv_0041" => "editor.show_object_info",
    "gv_0042" => "increase_max_levels",
    "gv_0043" => "editor.effect_lines",
    "gv_0044" => "editor.trigger_boxes",
    "gv_0045" => "editor.debug_draw",
    "gv_0046" => "editor.hide_ui_on_test",
    "gv_0047" => "showed_profile_text",
    "gv_0048" => "viewed_own_profile",
    "gv_0049" => "editor.columns",
    "gv_0050" => "editor.rows",
    "gv_0051" => "showed_ng_message",
    "gv_0052" => "fast_respawn",
    "gv_0053" => "showed_free_games_popup",
    "gv_0055" => "check_if_server_online",
    "gv_0056" => "disable_high_object_alert",
    "gv_0057" => "editor.hold_to_swipe",
    "gv_0058" => "editor.duration_lines",
    "gv_0059" => "editor.swipe_cycle_mode",
    "gv_0060" => "default_mini_icon",
    "gv_0061" => "switch_spider_teleport_colour",
    "gv_0062" => "switch_dash_fire_colour",
    "gv_0063" => "showed_unverified_coins_message",
    "gv_0064" => "editor.select_filter",
    "gv_0065" => "move_optimization",
    "gv_0066" => "high_capacity_mode",
    "gv_0067" => "high_start_pos_accuracy",
    "gv_0068" => "quick_checkpoint_mode",
    "gv_0069" => "comment_mode",
    "gv_0070" => "showed_unlisted_level_message",
    "gv_0071" => "hide_practice_buttons",
    "gv_0072" => "disable_gravity_effect",
    "gv_0073" => "new_completed_filter",
    "gv_0074" => "show_restart_button",
    "gv_0075" => "parental.disable_comments",
    "gv_0076" => "parental.disable_account_comments",
    "gv_0077" => "parental.featured_levels_only",
    "gv_0078" => "editor.hide_background",
    "gv_0079" => "editor.hide_grid_on_play",
    "gv_0081" => "disable_shake",
    "gv_0082" => "disable_high_object_alert",
    "gv_0083" => "disable_song_alert",
    "gv_0084" => "manual_order",
    "gv_0088" => "compact_comments",
    "gv_0089" => "extended_info_mode",
    "gv_0090" => "auto_load_comments",
    "gv_0091" => "created_level_folder",
    "gv_0092" => "saved_level_folder",
    "gv_0093" => "increase_levels_per_page",
    "gv_0094" => "more_comments",
    "gv_0095" => "do_not",
    "gv_0096" => "switch_wave_trail_colour",
    "gv_0097" => "editor.link_controls",
    "gv_0098" => "level_leaderboard_type",
    "gv_0099" => "show_leaderboard_percent",
    "gv_0100" => "practice_death_effect",
    "gv_0101" => "force_smooth_fix",
    "gv_0102" => "editor.editor_smooth_fix",
    "gv_0103" => "editor.layer_locking",
    "gv_0104" => "record_order",
    "gv_0105" => "startpos_playback",
    "gv_0106" => "show_meltdown_promo",
    "gv_0108" => "auto_low_detail",
    "gv_0109" => "level_info_label",
    "gv_0110" => "fast_editor_preview",
    "gv_0112" => "editor.increase_scale_limit",
    "gv_0113" => "flip_platformer_controls",
    "gv_0115" => "show_fps",
    "gv_0116" => "use_custom_fps",
    "gv_0117" => "preview_particle",
    "gv_0118" => "preview_animation",
    "gv_0119" => "dont_save_levels",
    "gv_0121" => "editor.hide_invisible",
    "gv_0122" => "disable_menu_music",
    "gv_0125" => "editor.unlock_practice_music",
    "gv_0126" => "decimal_percentage",
    "gv_0127" => "save_gauntlet_levels",
    "gv_0128" => "lock_cursor_in_game",
    "gv_0129" => "disable_portal_labels",
    "gv_0130" => "orb_labels",
    "gv_0131" => "editor.smart_template.use_nearby_as_reference",
    "gv_0132" => "editor.smart_template.dont_delete",
    "gv_0133" => "editor.group_id_filter",
    "gv_0134" => "hide_attempts",
    "gv_0135" => "hide_attempts_practice",
    "gv_0136" => "extra_ldm",
    "gv_0137" => "editor.hide_particle_icons",
    "gv_0139" => "editor.colour_channel_filter",
    "gv_0140" => "disable_orb_scale",
    "gv_0141" => "disable_trigger_orb_scale",
    "gv_0142" => "reduce_audio_quality",
    "gv_0144" => "audio_visualizer",
    "gv_0145" => "show_time",
    "gv_0146" => "disable_checkpoints",
    "gv_0147" => "editor.sfx_search_folder_name",
    "gv_0148" => "editor.sfx_compact_mode",
    "gv_0149" => "editor.show_clicks",
    "gv_0150" => "editor.auto_pause",
    "gv_0151" => "editor.start_optimization",
    "gv_0152" => "editor.hide_path",
    "gv_0153" => "explode_player_on_death",
    "gv_0155" => "disable_shader_anti_aliasing",
    "gv_0156" => "editor.disable_paste_state_groups",
    "gv_0157" => "level_upload_guidelines_shown",
    "gv_0158" => "editor.preview_shaders",
    "gv_0159" => "audio_fix_01",
    "gv_0163" => "quick_keys",
    "gv_0164" => "level_leaderboard_mode",
    "gv_0166" => "practice_mode_hitboxes",
    "gv_0167" => "confirm_exit",
    "gv_0168" => "fast_menu",
    "gv_0169" => "editor.small_warp_buttons",
    "gv_0170" => "borderless_fullscreen",
    "gv_0171" => "disable_player_hitbox",
    "gv_0172" => "disable_shake",
    "gv_0173" => "editor.playtesting_ignore_damage",
    "gv_0174" => "hide_playtest_text"
};

pub const GAME_EVENTS: Map<&'static str, &'static str> = phf_map! {
    "ugv_1" => "challenge_unlocked",
    "ugv_2" => "glubfub_hint_1",
    "ugv_3" => "glubfub_hint_2",
    "ugv_4" => "challenge_completed",
    "ugv_5" => "treasure_room_unlocked",
    "ugv_6" => "chamber_of_time_unlocked",
    "ugv_7" => "chamber_of_time_discovered",
    "ugv_8" => "found_master_emblem",
    "ugv_9" => "gatekeeper_dialogue",
    "ugv_10" => "scratch_dialogue",
    "ugv_11" => "scratch_shop_unlocked",
    "ugv_12" => "monster_dialogue",
    "ugv_13" => "demon_gauntlet_key",
    "ugv_14" => "blue_key",
    "ugv_15" => "green_key",
    "ugv_16" => "orange_key",
    "ugv_17" => "shopkeeper_dialogue",
    "ugv_18" => "gdw_online_unlocked",
    "ugv_19" => "monster_encountered",
    "ugv_20" => "community_shop_unlocked",
    "ugv_21" => "potbor_dialogue",
    "ugv_22" => "youtube_chest",
    "ugv_23" => "facebook_chest",
    "ugv_24" => "twitter_chest",
    "ugv_25" => "explorers_unlocked",
    "ugv_26" => "twitch_chest",
    "ugv_27" => "discord_chest",
    "ugv_28" => "tower_clicked",
    "ugv_29" => "tower_entered",
    "ugv_30" => "accepted_tos",
    "ugv_31" => "zolguroth_encountered",
    "ugv_32" => "reddit_chest",
    "ugv_33" => "tower_floor_1_completed",
    "ugv_34" => "diamond_shop_unlocked",
    "ugv_35" => "mechanic_unlocked",
    "ugv_36" => "mechanic_dialogue",
    "ugv_37" => "diamond_shopkeeper_dialogue",
    "ugv_38" => "unknown_1"
};

pub const OFFICIAL_LEVEL_NAMES: Map<&'static str, &'static str> = phf_map! {
    "1" => "stereo_madness",
    "2" => "back_on_track",
    "3" => "polargeist",
    "4" => "dry_out",
    "5" => "base_after_base",
    "6" => "cant_let_go",
    "7" => "jumper",
    "8" => "time_machine",
    "9" => "cycles",
    "10" => "xstep",
    "11" => "clutterfunk",
    "12" => "theory_of_everything",
    "13" => "electroman_adventures",
    "14" => "clubstep",
    "15" => "electrodynamix",
    "16" => "hexagon_force",
    "17" => "blast_processing",
    "18" => "theory_of_everything_2",
    "19" => "geometrical_dominator",
    "20" => "deadlocked",
    "21" => "fingerdash",
    "22" => "dash",
    "23" => "explorers",
    "1001" => "the_seven_seas",
    "1002" => "viking_arena",
    "1003" => "airborne_robots",
    "2001" => "payload",
    "2002" => "beast_mode",
    "2003" => "machina",
    "2004" => "years",
    "2005" => "frontlines",
    "2006" => "space_pirates",
    "2007" => "striker",
    "2008" => "embers",
    "2009" => "round_1",
    "2010" => "monster_dance_off",
    "3001" => "the_challenge",
    "4001" => "press_start",
    "4002" => "nock_em",
    "4003" => "power_trip",
    "5001" => "the_tower",
    "5002" => "the_sewers",
    "5003" => "the_cellar",
    "5004" => "the_secret_hollow"
};

pub const WRAITH_KEYS: Map<&'static str, &'static str> = phf_map! {
    "o_secret_6" => "backondash",
    "o_secret_11" => "robtopisnice",
    "o_secret_12" => "skibidi",
    "o_secret_13" => "checksteam",
    "o_secret_14" => "fireinthehole",
    "o_secret_15" => "wellmet",
    "o_secret_16" => "wateronthehill",
    "o_secret_17" => "bussin",
    "o_secret_18" => "thickofit",
    "o_secret_19" => "touchgrass",
    "o_secret_21" => "key",
};
