mod constants;
mod util;

use constants::XOR_KEY;
use serde_json::Value;
use std::error::Error;
use std::process::exit;
use util::decrypt::*;
use util::file::*;
use util::xml::*;

fn main() {
    if let Some(data_encrypted) = get_dat_data() {
        let decrypt_data_windows = || -> Result<Vec<u8>, Box<dyn Error>> {
            Ok(gz_decompress(&decode_urlsafe_base64(&xor_decrypt(
                &data_encrypted,
                XOR_KEY,
            ))?))
        };

        let decrypt_data_mac =
            || -> Result<Vec<u8>, Box<dyn Error>> { Ok(decode_aes(&data_encrypted)) };

        let data: Vec<u8> = if let Ok(d) = decrypt_data_windows() {
            d
        } else if let Ok(d) = decrypt_data_mac() {
            d
        } else {
            println!("Decryption failed, corrupted save file?");
            exit(1);
        };

        match validate_xml(&data) {
            Ok(_) => {
                let parsed_data: Value = parse_save(&data);

                if let Some(output_path) = prompt_output_path() {
                    write_decrypted_json(&parsed_data, output_path);
                }
            }
            Err(e) => {
                println!("Decryption failed: {e}");
                exit(1);
            }
        }
    }
}
