use aes::cipher::{block_padding::Pkcs7, BlockDecryptMut, KeyInit};
use aes::Aes256;
use base64::{
    engine::general_purpose::{STANDARD, URL_SAFE},
    DecodeError, Engine,
};
use ecb::Decryptor;
use flate2::read::GzDecoder;
use std::io::Read;

use crate::constants::AES_KEY;

type Aes256Decryptor = Decryptor<Aes256>;

// Step 1
pub fn xor_decrypt(data: &[u8], key: u8) -> Vec<u8> {
    data.iter().map(|byte| byte ^ key).collect()
}

// Step 2
pub fn decode_urlsafe_base64(data: &[u8]) -> Result<Vec<u8>, DecodeError> {
    let url_safe_data: Vec<u8> = data
        .iter()
        .filter(|&&byte| byte != 0)
        .map(|&byte| match byte {
            b'+' => b'-',
            b'/' => b'_',
            _ => byte,
        })
        .collect();

    URL_SAFE.decode(url_safe_data)
}

// Step 3
pub fn gz_decompress(data: &[u8]) -> Vec<u8> {
    let mut decoder: GzDecoder<&[u8]> = GzDecoder::new(data);
    let mut decompressed: Vec<u8> = Vec::new();
    decoder.read_to_end(&mut decompressed).unwrap();
    decompressed
}

pub fn decode_base64(data: &str) -> String {
    STANDARD
        .decode(data)
        .unwrap()
        .iter()
        .map(|&byte| byte as char)
        .collect()
}

pub fn decode_aes(data: &[u8]) -> Vec<u8> {
    let mut buf: Vec<u8> = data.to_vec();
    Aes256Decryptor::new(&AES_KEY.into())
        .decrypt_padded_mut::<Pkcs7>(&mut buf)
        .unwrap();
    buf
}
