use dirs::{data_local_dir, desktop_dir};
use rfd::FileDialog;
use serde_json::{to_writer_pretty, Value};
use std::{fs::File, io::Read, path::PathBuf};

pub fn get_dat_data() -> Option<Vec<u8>> {
    match prompt_input_path() {
        Some(path) => {
            let mut file: File = File::open(path).unwrap();
            let mut data: Vec<u8> = Vec::new();

            file.read_to_end(&mut data).unwrap();

            Some(data)
        }
        None => None,
    }
}

pub fn prompt_input_path() -> Option<PathBuf> {
    FileDialog::new()
        .set_title("Select Geometry Dash Save")
        .set_file_name("CCGameManager.dat")
        .add_filter("Geometry Dash Save", &["dat"])
        .set_directory(data_local_dir().unwrap())
        .pick_file()
}

pub fn prompt_output_path() -> Option<PathBuf> {
    FileDialog::new()
        .set_title("Select Output Location")
        .set_file_name("CCGameManager.json")
        .add_filter("Geometry Dash Translated Save", &["json"])
        .set_directory(desktop_dir().unwrap())
        .save_file()
}

pub fn write_decrypted_json(data: &Value, path: PathBuf) {
    let output_file: File = File::create(path).expect("Failed to create file");
    to_writer_pretty(output_file, data).expect("Failed to write JSON");
}
